/***************************************************************************
 *   Copyright (C) 2004 by Diego 'Flameeyes' Petteno                       *
 *   dgp85@users.sourceforge.net                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qdir.h>
#include <qstringlist.h>
#include <qsplitter.h>
#include <qlayout.h>

#include <klocale.h>
#include <kconfig.h>
#include <kfiletreeview.h>
#include <kfileiconview.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <klistview.h>
#include <kaction.h>
#include <kmainwindow.h>
#include <kstatusbar.h>
#include <kprogress.h>
#include <kcmdlineargs.h>

#include "kdigestwidget.h"
#include "digestfile.h"
#include "sfvfile.h"
#include "md5sumfile.h"

kdigestWidget::kdigestWidget(QWidget* parent, const char* name, WFlags fl)
        : kdigestWidgetBase(parent,name,fl)
{
	QHBoxLayout *wl = new QHBoxLayout(this, 11, 6);

	m_splitter = new QSplitter(this, "splitter");
	m_splitter->setOpaqueResize(true);
	
	QLayout *sl = new QGridLayout(m_splitter);
	
	m_allDirs = new KFileTreeView(m_splitter, "allDirs");
        m_allDirs->addColumn( tr( "Directory" ) );
	m_allDirs->setMaximumSize(200, 32767);
        m_allDirs->setResizePolicy( QListView::AutoOneFit );
        m_allDirs->setRootIsDecorated( true );
	
	sl->add(m_allDirs);
	
	currentDir->setMaximumSize(32767, 32767);
	sfvContent->setMaximumSize(32767, 32767);
	fileFrame->setMaximumSize(32767, 32767);
	fileFrame->reparent(m_splitter, 0, QPoint(), true);
	
	m_splitter->setCollapsible(m_allDirs, false);
	m_splitter->setCollapsible(fileFrame, false);
	wl->add(m_splitter);
	
	KFileTreeBranch *home = m_allDirs->addBranch(
		QDir::home().path(),
		"Home",
		KGlobal::iconLoader()->loadIcon("folder_home", KIcon::Small)
	);
	m_allDirs->setDirOnlyMode(home, true);
	m_root = m_allDirs->addBranch(
		"file:/",
		"Root",
		KGlobal::iconLoader()->loadIcon("folder_red", KIcon::Small)
	);
	m_allDirs->setDirOnlyMode(m_root, true);
	
	// This is while we are *checking* files, but when we create new, we should pass to KIconView::Select
	currentDir->setMode(KIconView::Execute);

	connect( m_allDirs, SIGNAL( selectionChanged(QListViewItem*) ), this, SLOT( dirSelected() ) );
	connect( currentDir, SIGNAL( executed(QIconViewItem*) ), this, SLOT( fileExecuted(QIconViewItem*) ) );
	
	m_aCheck = new KAction(i18n("&Check files"), "run", 0, this, SLOT( checkFile() ), (reinterpret_cast<KMainWindow*>(parent))->actionCollection(), "checkfile" );
	m_aCheck->setEnabled(false);
	
	m_statusBar = (reinterpret_cast<KMainWindow*>(parent))->statusBar();

	m_statusBar->setSizePolicy( QSizePolicy( QSizePolicy::Ignored, QSizePolicy::Fixed ) );
	m_statusBar->setMinimumHeight(25);
	m_statusBar->setMaximumHeight(25);
	
	m_statusLabel = new QLabel(this);
	m_statusLabel->setSizePolicy( QSizePolicy( QSizePolicy::Minimum, QSizePolicy::Fixed ) );
	m_statusLabel->setMinimumHeight(25);
	m_statusLabel->setMaximumHeight(25);
	m_statusBar->addWidget(m_statusLabel);
	
	m_progress = new KProgress(this);
	m_progress->setEnabled(false);
	m_progress->setMaximumSize( QSize(75, 20) );
	m_statusBar->addWidget(m_progress, 0, true);

	m_file = NULL;
	
//	if ( KCmdLineArgs::parsedArgs()->isSet("filename") )
	{
		///@todo Open the file requested
	}
}

kdigestWidget::~kdigestWidget()
{
	delete m_file;
}

/**
@brief Changes the directory shown up on currentDir and update m_allDirs
@param newpath Path to move the interface to
*/
void kdigestWidget::changeDir(const KURL &newpath)
{
	currentDir->KFileView::clear();
	sfvContent->clear();

	QDir curr(newpath.path());
	QStringList el = curr.entryList();
	for(QStringList::iterator it = el.begin(); it != el.end(); it++)
	{
		if ( *it == "." || *it == ".." )
			continue;
		if ( (*it)[0] == '.' )
		{
			///@todo There should be an option to show/hide hidden files
			continue;
		}
		
		currentDir->insertItem( new KFileItem(
			KFileItem::Unknown,
			KFileItem::Unknown,
			newpath.url() + "/" + (*it)
			)
		);
	}
}

/**
@brief Slot called when the m_allDirs selection is changed
@param lvi KFileListViewItem selected
*/
void kdigestWidget::dirSelected()
{
	changeDir(m_allDirs->currentURL());
}

/**
@brief Slot called when is executed a file into currentDir
@param ivi KFileIconViewItem executed
*/
void kdigestWidget::fileExecuted(QIconViewItem* ivi)
{
	delete m_file;
	m_file = NULL;
	sfvContent->clear();
	
	KFileIconViewItem *fi = reinterpret_cast<KFileIconViewItem*>(ivi);
	
	const KURL url = fi->fileInfo()->url();
	
	if ( fi->fileInfo()->isDir() )
	{
		m_allDirs->currentKFileTreeViewItem()->setOpen(true);
		
		m_allDirs->setSelected(m_allDirs->currentKFileTreeViewItem()->branch()->findTVIByURL( url ), true);
		
		changeDir(url);
	} else if ( fi->fileInfo()->name().right(4).lower() == ".sfv" ) {
		m_file = new KDigest::SFVfile(url.path(), false, this, "sfvfile");
		rereadEntries();
	} else if (
			fi->fileInfo()->name() == "MD5SUM" ||
			fi->fileInfo()->name() == "MD5SUMS" ||
			fi->fileInfo()->name().right(4).lower() == ".md5"
		) {
		m_file = new KDigest::MD5SUMfile(url.path(), false, this, "md5sumfile");
		rereadEntries();
	}
	
	// Now update the actions....
	m_aCheck->setEnabled( m_file );
}

/**
@brief Clear and repopulate sfvContent with the entries found in m_file (setting the status right)
*/
void kdigestWidget::rereadEntries()
{
	// Check if called rightfully
	if ( ! m_file ) return;
	
	sfvContent->clear();
	
	DigestFile::DigestEntries es = m_file->entries();
	for ( DigestFile::DigestEntries::iterator it = es.begin(); it != es.end(); it++)
	{
		KListViewItem *lvi = new KListViewItem(
			sfvContent,
			sfvContent->lastItem(),
			(*it)->filename,
			(*it)->digest()
		);
		switch( (*it)->status )
		{
			case DigestFile::dsNotChecked:	// Do nothing
				break;
			case DigestFile::dsOk:		// Set the right icon and Digest
				lvi->setText(2, (*it)->actual());
				lvi->setPixmap(0, KGlobal::iconLoader()->loadIcon("button_ok", KIcon::Small));
				break;
			case DigestFile::dsBadDigest:	// Set the Digest...
				lvi->setText(2, (*it)->actual());
				lvi->setPixmap(0, KGlobal::iconLoader()->loadIcon("button_cancel", KIcon::Small));
				break;
			case DigestFile::dsNotFound:
				lvi->setText(2, "Not found");
				lvi->setPixmap(0, KGlobal::iconLoader()->loadIcon("button_cancel", KIcon::Small));
				break;
			case DigestFile::dsErrorAccess:
				lvi->setText(2, "Not accessible");
				lvi->setPixmap(0, KGlobal::iconLoader()->loadIcon("button_cancel", KIcon::Small));
				break;
			default:			// ..and then the wrong icon (for all other cases)
				lvi->setPixmap(0, KGlobal::iconLoader()->loadIcon("button_cancel", KIcon::Small));
		}
	}
}

/**
@brief Slot called when the Check file action is selected
*/
void kdigestWidget::checkFile()
{
	if ( ! m_file ) return;

	connect( m_file, SIGNAL(cycleCompleted(int)), m_progress, SLOT(setProgress(int)) );
	m_progress->setTotalSteps( m_file->entries().count() );
	m_progress->setEnabled(true);
	
	m_allDirs->setEnabled(false); currentDir->setEnabled(false);
	bool ok = m_file->testFile(m_allDirs->currentURL().path());
	m_allDirs->setEnabled(true); currentDir->setEnabled(true);
	
	disconnect( m_file );
	m_progress->setEnabled(false);
	
	rereadEntries();
	
	if ( ok )
		m_statusLabel->setText(tr("Digested files are all present and matches the given digest"));
	else
		m_statusLabel->setText(tr("Some digested files 't present, or doesn't matches the given digest, see the window for details"));
}

#include "kdigestwidget.moc"

